unit charmap;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids;

type
  TCharmapForm = class(TForm)
    SelectedFontLabel: TLabel;
    SelectedCharLabel: TLabel;
    SelectedCharacterID: TLabel;
    SelectedCharacter: TLabel;
    SelectionAllLabel: TLabel;
    CharGrid: TStringGrid;
    SelectedFont: TComboBox;
    BtnExit: TButton;
    BtnAdd: TButton;
    BtnClear: TButton;
    BtnCopy: TButton;
    SelectionAll: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure SelectedFontChange(Sender: TObject);
    procedure SelectCharacter(Which: integer);
    procedure CharGridSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure BtnClearClick(Sender: TObject);
    procedure BtnExitClick(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnCopyClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    function  Execute(Sender: TObject): AnsiString;
    { Public-Deklarationen }
  end;

var
  CharmapForm: TCharmapForm;
  SavedKey: integer;

implementation

{$R *.DFM}

procedure TCharmapForm.FormCreate(Sender: TObject);
var i,x,y: integer;
begin

    // Write all available fonts to SelectedFont ComboBox
    for i := 0 to Screen.Fonts.Count-1 do
        SelectedFont.Items.Add(Screen.Fonts.Strings[i]);

    // Write characters 32 to 255 into CharGrid
    for x := 0 to 31 do for y := 0 to 6 do
        CharGrid.Cells[x,y] := AnsiString(char(byte(x+32*(y+1))));

    // Use Times New Roman as standard font
    SelectedFont.ItemIndex := SelectedFont.Items.IndexOf('Times New Roman');
    SelectedFontChange(Sender);

    // Clear results from former calls
    BtnClearClick(Sender);
end;

function TCharmapForm.Execute(Sender: TObject): AnsiString;
begin
    // NOTE: If you use TCharmap as a dialog window, please call Sender
    // function to run the dialog and wait for its result, which is an
    // AnsiString representing all characters chosen from the char map.

    BtnClearClick(Sender);
    SelectedFont.ItemIndex := SelectedFont.Items.IndexOf('Times New Roman');
    SelectedFontChange(Sender);

    // Wait for exit click
    ShowModal();

    Execute := SelectionAll.Text;
end;

procedure TCharmapForm.SelectedFontChange(Sender: TObject);
begin
    // Another font was selected
    CharGrid.Font.Name := SelectedFont.Items.Strings[SelectedFont.ItemIndex];
    SelectedCharacter.Font.Name := SelectedFont.Items.Strings[SelectedFont.ItemIndex];
    SelectionAll.Font.Name := SelectedFont.Items.Strings[SelectedFont.ItemIndex];
end;

procedure TCharmapForm.SelectCharacter(Which: integer);
var ShortKeyNumber: AnsiString;
begin
    // Show selected character
    SelectedCharacter.Caption := AnsiString(char(byte(Which)));

    // Show alternative key code
    ShortKeyNumber := IntToStr(Which);
    while (Length(ShortKeyNumber) < 4) do
        ShortKeyNumber := '0' + ShortKeyNumber;

    // Determine if key code is valid and save key
    if (Which = 0)
    then SelectedCharacterID.Caption := '( None )'
    else SelectedCharacterID.Caption := '( ALT + ' + ShortKeyNumber + ' )';
    SavedKey := Which;
end;

procedure TCharmapForm.CharGridSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
    // Select character from grid
    SelectCharacter(ACol + 32*(ARow+1));
end;

procedure TCharmapForm.BtnClearClick(Sender: TObject);
var Rect: TGridRect;
begin
    // Reset All
    SelectCharacter(0);
    Rect.Left := -1;
    Rect.Top := -1;
    Rect.Right := -1;
    Rect.Bottom := -1;
    CharGrid.Selection := Rect;
    SelectCharacter(0);
    SelectionAll.Text := '';
end;

procedure TCharmapForm.BtnExitClick(Sender: TObject);
begin
    // Exit. If the char map was started as a dialog, Close()
    // will go back to Execute().
    Close();
end;

procedure TCharmapForm.BtnAddClick(Sender: TObject);
begin
    // Copy currently selected character to SelectionAll label
    if (SavedKey > 0) then
        SelectionAll.Text := SelectionAll.Text + char(SavedKey);
end;

procedure TCharmapForm.BtnCopyClick(Sender: TObject);
begin
    // If any characters were selected, copy selection to Clipboard
    if (Length(SelectionAll.Text) > 0) then
    begin
        // Note: Only selected characters are copied, so select the
        // complete text first !
        SelectionAll.SelectAll();
        SelectionAll.CopyToClipboard();
    end;
end;



end.
